@@ -51,10 +51,10 @@ BUFFER_SIZE = 8 * 1024
 
 @attr.s
 class HealthCheckState:
-    valid_examples = attr.ib(default=0)
-    invalid_examples = attr.ib(default=0)
-    overrun_examples = attr.ib(default=0)
-    draw_times = attr.ib(default=attr.Factory(list))
+    valid_examples: int = attr.ib(default=0)
+    invalid_examples: int = attr.ib(default=0)
+    overrun_examples: int = attr.ib(default=0)
+    draw_times: list = attr.ib(factory=list)
 
 
 class ExitReason(Enum):
@@ -80,6 +80,7 @@ class ConjectureRunner:
     def __init__(
         self,
         test_function,
+        *,
         settings=None,
         random=None,
         database_key=None,
@@ -549,7 +550,7 @@ class ConjectureRunner:
             self.statistics["targets"] = dict(self.best_observed_targets)
         self.debug(f"exit_with({reason.name})")
         self.exit_reason = reason
-        raise RunIsComplete()
+        raise RunIsComplete
 
     def should_generate_more(self):
         # End the generation phase where we would have ended it if no bugs had
@@ -975,11 +976,11 @@ class ConjectureRunner:
             self,
             example,
             predicate,
-            allow_transition,
+            allow_transition=allow_transition,
             explain=Phase.explain in self.settings.phases,
         )
 
-    def cached_test_function(self, buffer, error_on_discard=False, extend=0):
+    def cached_test_function(self, buffer, *, error_on_discard=False, extend=0):
         """Checks the tree to see if we've tested this buffer, and returns the
         previous result if we have.
 
@@ -1014,7 +1015,7 @@ class ConjectureRunner:
 
             class DiscardObserver(DataObserver):
                 def kill_branch(self):
-                    raise ContainsDiscard()
+                    raise ContainsDiscard
 
             observer = DiscardObserver()
         else:
