@@ -9,14 +9,13 @@
 # obtain one at https://mozilla.org/MPL/2.0/.
 
 import datetime as dt
-import os.path
 from calendar import monthrange
 from functools import lru_cache
 from importlib import resources
+from pathlib import Path
 from typing import Optional
 
 from hypothesis.errors import InvalidArgument
-from hypothesis.internal.conjecture import utils
 from hypothesis.internal.validation import check_type, check_valid_interval
 from hypothesis.strategies._internal.core import sampled_from
 from hypothesis.strategies._internal.misc import just, none
@@ -113,9 +112,9 @@ def draw_capped_multipart(
         if name == "day" and not cap_high:
             _, high = monthrange(**result)
         if name == "year":
-            val = utils.integer_range(data, low, high, 2000)
+            val = data.draw_integer(low, high, shrink_towards=2000)
         else:
-            val = utils.integer_range(data, low, high)
+            val = data.draw_integer(low, high)
         result[name] = val
         cap_low = cap_low and val == low
         cap_high = cap_high and val == high
@@ -125,7 +124,7 @@ def draw_capped_multipart(
         # the logic above, and be very sensitive to the specific timezone
         # (at the cost of efficient shrinking and mutation), so at least for
         # now we stick with the status quo and generate it independently.
-        result["fold"] = utils.integer_range(data, 0, 1)
+        result["fold"] = data.draw_integer(0, 1)
     return result
 
 
@@ -312,7 +311,7 @@ class TimedeltaStrategy(SearchStrategy):
         for name in ("days", "seconds", "microseconds"):
             low = getattr(self.min_value if low_bound else dt.timedelta.min, name)
             high = getattr(self.max_value if high_bound else dt.timedelta.max, name)
-            val = utils.integer_range(data, low, high, 0)
+            val = data.draw_integer(low, high)
             result[name] = val
             low_bound = low_bound and val == low
             high_bound = high_bound and val == high
@@ -342,7 +341,7 @@ def timedeltas(
 def _valid_key_cacheable(tzpath, key):
     assert isinstance(tzpath, tuple)  # zoneinfo changed, better update this function!
     for root in tzpath:
-        if os.path.exists(os.path.join(root, key)):  # pragma: no branch
+        if Path(root).joinpath(key).exists():  # pragma: no branch
             # No branch because most systems only have one TZPATH component.
             return True
     else:  # pragma: no cover
